\NeedsTeXFormat{LaTeX2e}[1995/12/01]
\ProvidesClass{exam}[2016/02/05 v1.11 Non Standard LaTeX document
  class]

%%----------------------------------------------------------------
%%  OPTIONS: All options should be declared and defined in this
%%  section.
%%----------------------------------------------------------------

\newif\if@constants
\@constantsfalse

\newif\if@marktot
\@marktotfalse

\newif\if@sffonts
\@sffontsfalse

\newif\if@ptmfonts
\@ptmfontsfalse

\newlength{\markbreak}

\DeclareOption{constants}{ \@constantstrue }
\DeclareOption{marks}{ \@marktottrue }
\DeclareOption{sanserif}{ \@sffontstrue }
\DeclareOption{timesroman}{ \@ptmfontstrue }
\DeclareOption*{%
  \PackageWarning{exam}{Unknown option `\CurrentOption'}%
}
\ProcessOptions\relax

\LoadClass[12pt]{extarticle}
\RequirePackage{graphicx}

\RequirePackage{xkeyval}

\RequirePackage[a4paper, textheight=195mm, textwidth=165mm, footskip=20mm,
  hmargin=.75in]{geometry}

\if@marktot
\setlength{\hoffset}{-.25in}
\addtolength{\marginparwidth}{.25in}
\else
%
\fi

\RequirePackage{xifthen}
\RequirePackage{fancyhdr}
\RequirePackage{lastpage}
\RequirePackage{ifpdf} 
\RequirePackage{setspace}
\RequirePackage{chngcntr} % counters in the document
\RequirePackage{marginnote}
\RequirePackage{linegoal}
\RequirePackage{xcolor}
\RequirePackage{amsmath,amssymb} % usual maths packages

\if@sffonts
  \renewcommand{\mathbf}[1]{\ensuremath\textbf{\textsf{#1}}}
  \RequirePackage[cmbright]{sfmath}
  %\RequirePackage{sfmath}		%old
  \renewcommand{\rmdefault}{phv}
  \renewcommand{\sfdefault}{phv}
  \renewcommand{\mathbf}[1]{\ensuremath\textbf{\textit{\textsf{#1}}}}
\else
  \if@ptmfonts
    \renewcommand{\rmdefault}{ptm}	%old
    \renewcommand{\sfdefault}{ptm}	%old
  \else
    \RequirePackage{newtxmath}		%new - not working properly for NRB
  \fi
\fi


\ifpdf
\DeclareGraphicsExtensions{.pdf,.png,.jpg,.mps}
\else
\DeclareGraphicsExtensions{.eps}
\fi

\renewcommand{\headrulewidth}{0pt}
\renewcommand{\footrulewidth}{0pt}
\setlength{\parskip}{12pt}     		
\setlength{\parindent}{0pt}

\pagestyle{fancy}
\lhead{}
\chead{}
\rhead{}
\lfoot{}
\rfoot{}

% set centre footnote
\cfoot{\bf \ifthenelse{\value{page} = \pageref{LastPage}}{\mbox{END OF
      PAPER}\\[1cm]({\@examiner})\\[1cm]}{%
    \ifthenelse{\value{page}=1}{~\\[2.5cm]}{PLEASE TURN OVER\\[2.5cm]}
  }
  {\small Class No.\ {%
      \ifx\@classcodeB\@empty
      \@classcode
      \else
      \@classcode~/~\@classcodeB
      \fi
  }}\hfill{} {\small Page \thepage{} of
    \pageref{LastPage}} }

\newcommand{\@examlength}{3 hours}
\newcommand{\duration}[1]{\renewcommand{\@examlength}{#1}}

\newcommand{\examtime}[1]{\def\@examtime{#1}}
\examtime{}

\newcommand{\examdate}[1]{\def\@examdate{#1}}
\examdate{}

\newcommand{\class}[2]{\def\@classcode{#1}\def\@classname{#2}}
\class{}{} \newcommand{\classes}[4]{ \def\@classcode{#1}
  \def\@classname{#2} \def\@classcodeB{#3} \def\@classnameB{#4} }
\classes{}{}{}{}

\newcommand{\examiner}[1]{\def\@examiner{#1}}
\examiner{}

\newcommand{\instructions}[1]{\def\@instructions{#1}}
\instructions{}

% Test whether users have set the required variables in preamble
\newcommand\checkpreamble{
  \ifx\@examtime\@empty
  \@latex@error{Exam starting time not defined in preamble}
  \fi
  \ifx\@examdate\@empty
  \@latex@error{Exam date not defined in preamble}
  \fi
  \ifx\@classcode\@empty
  \@latex@error{Class code not defined in preamble}
  \fi
  \ifx\@classname\@empty
  \@latex@error{Class name not defined in preamble}
  \fi
  \ifx\@examiner\@empty
  \@latex@error{Examiner name not defined in preamble}
  \fi
  \ifx\@instructions\@empty
  \@latex@error{Exam instructions not defined in preamble}
  \fi
}
% Check preamble variables right after \begin{document}
\AtBeginDocument{
  \checkpreamble
}

\newcounter{paperpart} % counter for paper part
\newcounter{questions} % individual questions
\counterwithout*{questions}{paperpart} % no-reset on changing sections
\newcounter{subquestions}
\newcounter{subsubquestions}

\newcounter{partmark}[paperpart]     % counter for total mark in part of paper
\newcounter{questionmark}[questions] % counter for total marks over a question

\newlength{\remainder}

\setlength{\markbreak}{-.45cm}

\providecommand*\@nameundef[1]{%
  \expandafter\let\csname #1\endcsname\@undefined}

% Borrowed from mdwlist.sty, therefore
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
\def\suspend{\@dblarg\suspend@i}
\def\suspend@i[#1]#2{%
  \def\suspended{abc}
  \edef\@tempa{%
    \noexpand\end{#2}%
    \def\expandafter\noexpand\csname resume.#1\endcsname{%
      \csname c@\@listctr\endcsname\the\csname c@\@listctr\endcsname%
    }%
  }%
  \@tempa%
}
\def\resume{\@dblarg\resume@i}
\def\resume@i[#1]#2{\@testopt{\resume@ii{#1}{#2}}{}}
\def\resume@ii#1#2[#3]{%
  \begin{#2}#3%
  \@ifundefined{resume.#1}{\ml@err@resume}{\@nameuse{resume.#1}}%
  \@nameundef{suspended}
}

% Put marks flush right
\def\marks#1{%
  \setlength{\remainder}{\linegoal}% Store remaining line dimension
  \ifthenelse{\lengthtest{\the\remainder<\the\markbreak}}{\newline}{}
  \ifmmode {%
    \ifinalign@ {% ams alignment environment
      \if@sffonts {% san serife mode
        \tag*{\textbf{\textsf[\,#1\,]}}
      } \else {% usual font
        \tag*{\textbf{[\,#1\,]}}
      }
      \fi
    } \else {% normal maths mode
      \if@sffonts {% san serife mode
        \tag*{\textbf{\textsf[\,#1\,]}}\\
      } \else {% usual font
        \tag*{\textbf{[\,#1\,]}}\\ }
      \fi
    }
    \fi
  } \else {%
    \hspace*{\fill}\textbf{[\,#1\,]}\\}
  \fi %
  \addtocounter{partmark}{#1} % running total if required
  \addtocounter{questionmark}{#1} % running total if required
}

\def\question{\item}

% define environment for paper section

\newenvironment{Section}[1][]{%
  \clearpage
  \refstepcounter{paperpart}
  \ifthenelse{\isempty{#1}}%
             {%
               \centerline{\bf{\large Section
                   \Alph{paperpart}}}
             }
             {%
               \centerline{\bf{\large Section
                   \Alph{paperpart} -- #1}}
             }
}{%
  \if@marktot
  \marginpar{\bfseries Part total \newline [\arabic{partmark}]}
  \fi
}

\newenvironment{questions}{%
  \begin{list}{ %               %%%% Begin list item label code
      \refstepcounter{questions}
      %
      \bfseries \upshape\arabic{questions}.
    }{ %        %%%% Begin list item body code      
      \setlength{\labelwidth}{3em}
      \setlength{\leftmargin}{0.06\textwidth}
      \setlength{\labelsep}{0.5ex}
      \setlength{\itemsep}{5ex}
      \singlespacing
    } %%%%% End list item body code
}{ %%%%% Begin wrapup environment code
  \end{list}
}

%%%% ENVIRONMENT FOR THE LIST OF CHOICES FOR A QUESTION %%%%
\newenvironment{subquestions}{ %%%%% begin preliminary environment code  
  \begin{list}{\bfseries \upshape\alph{subquestions})}{%     
      \renewcommand\thesubquestions{{\bfseries \upshape\alph{subquestions})}}
      \newcommand\labelsubquestions{\thesubquestions}
      \setlength{\topsep}{0.0ex}
      \setlength{\labelwidth}{3em}
      \setlength{\leftmargin}{0.035\textwidth}
      \setlength{\labelsep}{0.5ex}
      \setlength{\itemsep}{1.6ex}
      \setlength{\parsep}{0ex}
      \setstretch{0.8}
      \setlength{\listparindent}{0.1\textwidth}
      \singlespacing
      \usecounter{subquestions}
  }}{%
  \if@marktot
  \@ifundefined{suspended} {
  \marginpar{\bfseries Qn.\ total \newline [\arabic{questionmark}]}
  }{ %
  %\hspace*{1ex} \noindent
  %\marginpar{\bfseries Qn.\ So far \newline [\arabic{questionmark}]} 
  } \fi
\end{list}}

\newenvironment{subsubquestions}{ %%%%% begin preliminary environment code
  \begin{list}{\bfseries \upshape\roman{subsubquestions})}{%
      \renewcommand\thesubsubquestions{{\bfseries \upshape\roman{subsubquestions})}}
      \newcommand\labelsubsubquestions{\thesubsubquestions}
      \setlength{\topsep}{-0.3ex}
      \setlength{\labelwidth}{3em}
      \setlength{\leftmargin}{0.035\textwidth}
      \setlength{\labelsep}{0.5ex}
      \setlength{\itemsep}{0.6ex}
      \setlength{\parsep}{0ex}
      \setstretch{0.8}
      \setlength{\listparindent}{0.1\textwidth}
      \singlespacing
      \usecounter{subsubquestions}
}}{\end{list}}


\AtBeginDocument{
  
  \vspace*{-5cm}
  \centerline{\includegraphics[width=.225\textwidth]{logo}}
  
  \vspace*{-2ex}
  \begin{center}
    \bfseries{\Large Department of Physics} \\[10pt]
    
    \vspace*{1ex}
    % Course code and name
    \ifx\@classcodeB\@empty
    \bfseries{\large {\@classcode} {\@classname}} \\[18pt]
    \else
    \bfseries{\large {\@classcode} {\@classname}\\[.5ex]
      \large {\@classcodeB} {\@classnameB}} \\[16pt]
    \fi
    
    % Date and time
    \bfseries{\large {\@examdate} \hfill {\@examtime} -- {\@examlength}}\\[18pt]
    
    \vspace*{1ex}
    % instructions
    \bfseries{\large {\@instructions}}\\[12pt]
    
    \vspace*{.5ex} \if@sffonts \bfseries{ {\textbf Calculators must not be used
        to store text and/or formulae nor be capable of communication.
        Invigilators may require calculators to be reset.}}  \else
    \bfseries{\textit {\textbf Calculators must not be used to store text and/or
        formulae nor be capable of communication.  Invigilators may require
        calculators to be reset.}}  \fi
    
  \end{center}
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%        Physical constants        %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  \if@constants
  \bigskip
  \vspace*{-0.05\textheight}
  \begin{center}
    \begin{small}
      \underline{\normalsize Physical constants}\\[2.5ex]
      \begin{tabular}{l@{\;}l@{\hspace*{0.015\textwidth}}l@{\;}l}
        Speed of light & $c = 3.00 \times 10^8$\;m\,s$^{-1}$ & Bohr
        radius & $a_0 = 5.29 \times 10^{-11}$\;m\\
        
        Free space permittivity & $\epsilon_0 = 8.85 \times
        10^{-12}$\;F\,m$^{-1}$ & Compton wavelength &
        $\lambda_C = 2.43 \times 10^{-12}$\;m\\
        
        Free space permeability & $\mu_0 = 4\pi \times
        10^{-7}$\;H\,m$^{-1}$ & Avogadro's number & $N_A = 6.02 \times
        10^{23}$\;mol$^{-1}$\\
        
        Electrostatic constant & $k_E = 8.99 \times
        10^9$\;m\,F$^{-1}$ & Boltzmann's constant & $k_B = 1.38
        \times 10^{-23}$\;J\,K$^{-1}$\\
        
        Electron mass & $m_e = 9.11 \times 10^{-31}$\;kg & Universal
        Gas constant & $R = 8.31$\;J\,mol$^{-1}$\,K$^{-1}$\\
        
        Electron charge size & $e = 1.60 \times 10^{-19}$\;C &
        Gravity acc.\ on earth & $g = 9.81$\;m\,s$^{-2}$\\
        
        Planck's constant & $h = 6.63 \times 10^{-34}$\;J\,s &
        Gravitational constant & $G = 6.67 \times
        10^{-11}$\;N\,m$^2$\,kg$^{-2}$\\
        
        Wien's law constant & $b = 2.90 \times 10^{-3}$\;m\,K &
        Stefan-Boltzmann & $\sigma = 5.67 \times
        10^{-8}$\;W\,m$^{-2}$\,K$^{-4}$\\
        
        Rydberg constant & $R_\infty = 1.10 \times 10^7$\;m$^{-1}$ &
        Atomic mass unit & $u = 1.66 \times 10^{-27}$\;kg
      \end{tabular}
    \end{small}
  \end{center}
  \clearpage
  \fi
}

%\AtEndDocument{%
%}
